import json

import httpx
from fastavro import _schema_common, parse_schema

from ._types import JsonDict
from .exceptions import InvalidSchema, JsonRequired


def get_resource_from_url(url: str) -> JsonDict:
    try:
        response = httpx.get(url)
        return response.json()
    except json.JSONDecodeError as exc:
        raise JsonRequired(f"Can not convert to json the resource from {url}") from exc


def get_resource_from_path(path: str) -> JsonDict:
    with open(path, mode="r") as resource:
        schema = resource.read()
        try:
            return json.loads(schema)
        except json.JSONDecodeError as exc:
            raise JsonRequired(
                f"Can not convert to json the resource from {path}"
            ) from exc


def validate(*, schema: JsonDict) -> bool:
    try:
        parse_schema(schema=schema)
        return True
    except _schema_common.SchemaParseException as exc:
        raise InvalidSchema(
            f"Schema {schema} is not valid.\n Error: `{str(exc)}`"
        ) from exc
    except _schema_common.UnknownType as exc:
        raise InvalidSchema(
            f"Schema {schema} is an unknown type.\n Make sure that its type is a python dictionary"
        ) from exc



import ast
from typing import Dict, Optional

import rich
import typer
from dataclasses_avroschema import BaseClassEnum, ModelGenerator, serialization
from deepdiff import DeepDiff

from . import _schema_utils
from ._types import JsonDict, SerializationType

app = typer.Typer()
console = rich.console.Console()


def generate_error_messages(
    path_name: Optional[str] = "--path", url_name: Optional[str] = "--url"
) -> Dict[str, str]:
    return {
        "all_specified": f"You can not specicy both {path_name} and {url_name}",
        "required": f"{path_name} or {url_name} must be specified",
    }


def get_resource(
    *,
    path: Optional[str] = None,
    url: Optional[str] = None,
    error_messages: Optional[Dict[str, str]] = None,
) -> JsonDict:
    error_messages = error_messages or generate_error_messages()

    if all([path, url]):
        raise typer.BadParameter(error_messages["all_specified"])
    elif path is not None:
        return _schema_utils.get_resource_from_path(path=path)
    elif url is not None:
        return _schema_utils.get_resource_from_url(url=url)
    else:
        raise typer.BadParameter(error_messages["required"])


@app.command()
def generate_model(
    path: str = typer.Option(None),
    url: str = typer.Option(None),
    base_class: BaseClassEnum = typer.Option(
        BaseClassEnum.AVRO_MODEL,
        help="Model base class",
    ),
):
    resource = get_resource(path=path, url=url)
    _schema_utils.validate(schema=resource)

    model_generator = ModelGenerator(base_class=base_class.value)
    result = model_generator.render(schema=resource)
    console.print(result)


@app.command()
def generate_fake(
    path: str = typer.Option(None),
    url: str = typer.Option(None),
):
    print("Hello!!")


@app.command()
def schema_diff(
    source_path: str = typer.Option(None, help="Source path to the local schema"),
    source_url: str = typer.Option(None, help="Source schema url"),
    target_path: str = typer.Option(None, help="Target path to the local schema"),
    target_url: str = typer.Option(None, help="Target schema url"),
):
    source_resource = get_resource(
        path=source_path,
        url=source_url,
        error_messages=generate_error_messages(
            path_name="--source-path", url_name="--source-url"
        ),
    )
    target_resource = get_resource(
        path=target_path,
        url=target_url,
        error_messages=generate_error_messages(
            path_name="--path-path", url_name="--path-url"
        ),
    )
    _schema_utils.validate(schema=source_resource)
    _schema_utils.validate(schema=target_resource)

    console.print(DeepDiff(source_resource, target_resource))


@app.command()
def serialize(
    data: str = typer.Argument(None, callback=ast.literal_eval),
    path: str = typer.Option(None),
    url: str = typer.Option(None),
    serialization_type: SerializationType = typer.Option(
        SerializationType.AVRO,
    ),
):
    resource = get_resource(path=path, url=url)
    _schema_utils.validate(schema=resource)

    output = serialization.serialize(
        data, resource, serialization_type=serialization_type
    )
    console.print(output)


@app.command()
def deserialize(
    event: str,
    path: str = typer.Option(None),
    url: str = typer.Option(None),
    serialization_type: SerializationType = typer.Option(
        SerializationType.AVRO,
    ),
):
    resource = get_resource(path=path, url=url)
    _schema_utils.validate(schema=resource)

    data = (
        ast.literal_eval(event)
        if serialization_type == SerializationType.AVRO.value
        else event.encode()
    )
    output = serialization.deserialize(
        data, resource, serialization_type=serialization_type
    )
    console.print(output)


@app.command()
def validate_schema(
    path: str = typer.Option(None, help="Path to the local schema"),
    url: str = typer.Option(None, help="Schema url"),
):
    resource = get_resource(path=path, url=url)

    if _schema_utils.validate(schema=resource):
        console.print("[bold green]Valid schema!![/bold green] :+1: \n")
        console.print(resource)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dc_avro']

package_data = \
{'': ['*']}

install_requires = \
['aiofiles>=22.1.0,<23.0.0',
 'dataclasses-avroschema==0.37.1',
 'deepdiff>=6.2.3,<7.0.0',
 'httpx>=0.23.3,<0.24.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['dc-avro = dc_avro.main:app']}

setup_kwargs = {
    'name': 'dc-avro',
    'version': '0.6.1',
    'description': '',
    'long_description': "# Dataclasses Avro Schema CLI\n\nCommand line interface from [dataclasses-avroschema](https://github.com/marcosschroh/dataclasses-avroschema) to work with `avsc` resources\n\n[![Tests](https://github.com/marcosschroh/dc-avro/actions/workflows/tests.yaml/badge.svg)](https://github.com/marcosschroh/dc-avro/actions/workflows/tests.yaml)\n[![GitHub license](https://img.shields.io/github/license/marcosschroh/dc-avro.svg)](https://github.com/marcosschroh/dc-avro/blob/master/LICENSE)\n[![codecov](https://codecov.io/gh/marcosschroh/dc-avro/branch/master/graph/badge.svg)](https://codecov.io/gh/marcosschroh/dc-avro)\n![python version](https://img.shields.io/badge/python-3.7%2B-yellowgreen)\n\n## Requirements\n\n`python 3.7+`\n\n## Documentation\n\nhttps://marcosschroh.github.io/dc-avro/\n\n## Usage\n\nYou can validate `avro schemas` either from a `local file` or `url`:\n\nAssuming that we have a local file `schema.avsc` that contains an `avro schema`, we can check whether it is valid\n\n```bash\ndc-avro validate-schema --path schema.avsc\n\nValid schema!! 👍 \n\n{\n    'type': 'record',\n    'name': 'UserAdvance',\n    'fields': [\n        {'name': 'name', 'type': 'string'},\n        {'name': 'age', 'type': 'long'},\n        {'name': 'pets', 'type': {'type': 'array', 'items': 'string', 'name': 'pet'}},\n        {'name': 'accounts', 'type': {'type': 'map', 'values': 'long', 'name': 'account'}},\n        {'name': 'favorite_colors', 'type': {'type': 'enum', 'name': 'FavoriteColor', 'symbols': ['BLUE', 'YELLOW', 'GREEN']}},\n        {'name': 'has_car', 'type': 'boolean', 'default': False},\n        {'name': 'country', 'type': 'string', 'default': 'Argentina'},\n        {'name': 'address', 'type': ['null', 'string'], 'default': None},\n        {'name': 'md5', 'type': {'type': 'fixed', 'name': 'md5', 'size': 16}}\n    ]\n}\n```\n\nTo see all the commands execute `dc-avro --help`\n\n## Features\n\n* [x] Validate `schemas`\n* [x] Generate `models` from `schemas`\n* [x] Data deserialization with `schema`\n* [x] Data serialization with `schema`\n* [x] View diff between `schemas`\n* [] Generate fake data from `schema`\n\n## Development\n\n1. Install requirements: `poetry install`\n2. Code linting: `./scripts/format`\n3. Run tests: `./scripts/test`\n",
    'author': 'Marcos Schroh',
    'author_email': 'marcos.schroh@kpn.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
